/*
 * Copyright 2024 IBM Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import { LabIcon } from '@jupyterlab/ui-components';

import feedbackSvg from '../../style/icons/feedback.svg';
import qiskitSvg from '../../style/icons/Qiskit.svg';

export const feedbackIcon = new LabIcon({
  name: 'qiskit:feedback',
  svgstr: feedbackSvg
});

export const qiskitIcon = new LabIcon({
  name: 'qiskit:logo',
  svgstr: qiskitSvg
});
